@@ -1,8 +1,8 @@
 #' @export
 data_effects.btl <- function(x, data, ranef = empty_ranef(), 
                              prior = brmsprior(), knots = NULL, 
-                             nlpar = "", not4stan = FALSE, 
-                             smooths = NULL, gps = NULL, Jmo = NULL) {
+                             not4stan = FALSE, smooths = NULL, 
+                             gps = NULL, Jmo = NULL) {
   # prepare data for all types of effects for use in Stan
   # Args:
   #   data: the data passed by the user
@@ -18,28 +18,23 @@ data_effects.btl <- function(x, data, ranef = empty_ranef(),
   #   Jmo: optional precomputed values of Jmo for monotonic effects
   # Returns:
   #   A named list of data to be passed to Stan
-  nlpar <- check_nlpar(nlpar)
-  data_fe <- data_fe(x, data = data, knots = knots,
-                     nlpar = nlpar, not4stan = not4stan,
-                     smooths = smooths)
-  data_mo <- data_mo(x, data = data, ranef = ranef,
-                     prior = prior, Jmo = Jmo, nlpar = nlpar)
-  data_re <- data_re(ranef, data = data, nlpar = nlpar,
-                     not4stan = not4stan)
-  data_me <- data_me(x, data = data, nlpar = nlpar)
-  data_cs <- data_cs(x, data = data, nlpar = nlpar)
-  data_gp <- data_gp(x, data = data, nlpar = nlpar, gps = gps)
-  data_offset <- data_offset(x, data = data, nlpar = nlpar)
-  data_prior <- data_prior(prior, data = data, nlpar = nlpar)
-  c(data_fe, data_mo, data_re, data_me, data_cs, 
-    data_gp, data_offset, data_prior)
+  c(
+    data_fe(x, data, knots = knots, not4stan = not4stan, smooths = smooths),
+    data_mo(x, data, ranef = ranef, prior = prior, Jmo = Jmo),
+    data_re(x, data, ranef = ranef, not4stan = not4stan),
+    data_me(x, data),
+    data_cs(x, data),
+    data_gp(x, data, gps = gps),
+    data_offset(x, data),
+    data_prior(x, data, prior = prior)
+  )
 }
 
 #' @export 
 data_effects.btnl <- function(x, data, ranef = empty_ranef(), 
                               prior = brmsprior(), knots = NULL, 
-                              nlpar = "", not4stan = FALSE, 
-                              smooths = NULL, gps = NULL, Jmo = NULL) {
+                              not4stan = FALSE, smooths = NULL, 
+                              gps = NULL, Jmo = NULL) {
   # prepare data for non-linear parameters for use in Stan
   # matrix of covariates appearing in the non-linear formula
   out <- list()
@@ -49,14 +44,15 @@ data_effects.btnl <- function(x, data, ranef = empty_ranef(),
   }
   # fixes issue #127 occuring for factorial covariates
   colnames(C) <- all.vars(x$covars)
+  p <- usc(combine_prefix(x))
   if (not4stan) {
-    out <- c(out, nlist(C))
+    out[[paste0("C", p)]] <- C
   } else {
     # use vectors as indexing matrices in Stan is slow
     if (ncol(C)) {
       out <- c(out, setNames(
         as.list(as.data.frame(C)), 
-        paste0("C_", seq_len(ncol(C)))
+        paste0("C", p, "_", seq_len(ncol(C)))
       ))
     }
   }
@@ -64,22 +60,21 @@ data_effects.btnl <- function(x, data, ranef = empty_ranef(),
     out <- c(out,
       data_effects(
         x$nlpars[[nlp]], data, ranef = ranef,
-        prior = prior, knots = knots, nlpar = nlp,
-        not4stan = not4stan, smooths = smooths[[nlp]], 
-        gps = gps, Jmo = Jmo
+        prior = prior, knots = knots, not4stan = not4stan, 
+        smooths = smooths[[nlp]], gps = gps[[nlp]], 
+        Jmo = Jmo[[nlp]]
       )
     )
   }
   out
 }
 
-data_fe <- function(bterms, data, knots = NULL, nlpar = "", 
+data_fe <- function(bterms, data, knots = NULL,
                     not4stan = FALSE, smooths = NULL) {
   # prepare data for fixed effects for use in Stan 
   # Args: see data_effects
-  stopifnot(length(nlpar) == 1L)
   out <- list()
-  p <- usc(nlpar, "prefix")
+  p <- usc(combine_prefix(bterms))
   is_ordinal <- is_ordinal(bterms$family)
   is_bsts <- inherits(bterms$autocor, "cor_bsts")
   # the intercept is removed inside the Stan code for ordinal models
@@ -129,33 +124,36 @@ data_fe <- function(bterms, data, knots = NULL, nlpar = "",
     X <- structure(X, smooth_cols = scols, by_levels = by_levels)
     colnames(X) <- rename(colnames(X))
   }
-  avoid_auxpars(colnames(X), bterms = bterms)
+  avoid_dpars(colnames(X), bterms = bterms)
   c(out, setNames(list(ncol(X), X), paste0(c("K", "X"), p)))
 }
 
 data_mo <- function(bterms, data, ranef = empty_ranef(),
-                    prior = brmsprior(), nlpar = "",
-                    Jmo = NULL) {
+                    prior = brmsprior(), Jmo = NULL) {
   # prepare data for monotonic effects for use in Stan
   # Args: see data_effects
-  stopifnot(length(nlpar) == 1L)
-  p <- if (nchar(nlpar)) paste0("_", nlpar) else ""
+  px <- check_prefix(bterms)
+  p <- usc(combine_prefix(px))
   out <- list()
   if (is.formula(bterms[["mo"]])) {
-    Xmo <- prepare_mo_vars(bterms$mo, data, check = is.null(Jmo))
-    avoid_auxpars(colnames(Xmo), bterms = bterms)
+    Xmo <- mo_design_matrix(bterms$mo, data, check = is.null(Jmo))
+    avoid_dpars(colnames(Xmo), bterms = bterms)
     if (is.null(Jmo)) {
       Jmo <- as.array(apply(Xmo, 2, max))
     }
     out <- c(out, 
-      setNames(list(ncol(Xmo), Xmo, Jmo), paste0(c("Kmo", "Xmo", "Jmo"), p))
+      setNames(
+        list(ncol(Xmo), Xmo, Jmo), 
+        paste0(c("Kmo", "Xmo", "Jmo"), p)
+      )
     )
     # validate and assign vectors for dirichlet prior
     monef <- colnames(Xmo)
     for (i in seq_along(monef)) {
-      take <- prior$class == "simplex" & prior$coef == monef[i] & 
-        prior$nlpar == nlpar  
-      simplex_prior <- prior$prior[take]
+      simplex_prior <- subset2(prior,
+        class = "simplex", coef = monef[i], ls = px
+      )
+      simplex_prior <- simplex_prior$prior
       if (isTRUE(nzchar(simplex_prior))) {
         simplex_prior <- eval2(simplex_prior)
         if (length(simplex_prior) != Jmo[i]) {
@@ -171,20 +169,21 @@ data_mo <- function(bterms, data, ranef = empty_ranef(),
   out
 }
 
-data_re <- function(ranef, data, nlpar = "", not4stan = FALSE) {
+data_re <- function(bterms, data, ranef, not4stan = FALSE) {
   # prepare data for group-level effects for use in Stan
   # Args: see data_effects
-  stopifnot(length(nlpar) == 1L)
   out <- list()
-  take <- ranef$nlpar == nlpar & !ranef$type %in% c("mo", "me")
+  px <- check_prefix(bterms)
+  take <- find_rows(ranef, ls = px) &
+    !find_rows(ranef, type = c("mo", "me"))
   ranef <- ranef[take, ]
   if (nrow(ranef)) {
-    Z <- lapply(ranef[!duplicated(ranef$gn), ]$form, 
-                get_model_matrix, data = data)
+    unique_forms <- ranef[!duplicated(ranef$gn), "form"]
+    Z <- lapply(unique_forms, get_model_matrix, data = data)
     gn <- unique(ranef$gn)
     for (i in seq_along(gn)) {
-      r <- ranef[ranef$gn == gn[i], ]
-      idp <- paste0(r$id[1], usc(nlpar, "prefix"))
+      r <- subset2(ranef, gn = gn[i])
+      idp <- paste0(r$id[1], usc(combine_prefix(px)))
       if (isTRUE(not4stan)) {
         # for internal use in S3 methods
         if (ncol(Z[[i]]) == 1L) {
@@ -220,7 +219,7 @@ data_gr <- function(ranef, data, cov_ranef = NULL) {
   out <- list()
   ids <- unique(ranef$id)
   for (id in ids) {
-    id_ranef <- ranef[ranef$id == id, ]
+    id_ranef <- subset2(ranef, id = id)
     nranef <- nrow(id_ranef)
     group <- id_ranef$group[1]
     levels <- attr(ranef, "levels")[[group]]
@@ -232,8 +231,10 @@ data_gr <- function(ranef, data, cov_ranef = NULL) {
         scale <- isTRUE(attr(weights, "scale"))
         weights <- as.matrix(eval_rhs(weights, data))
         if (!identical(dim(weights), c(nrow(data), ngs))) {
-          stop2("Grouping structure 'mm' expects 'weights' to be a matrix ", 
-                "with as many columns as grouping factors.")
+          stop2(
+            "Grouping structure 'mm' expects 'weights' to be ", 
+            "a matrix with as many columns as grouping factors."
+          )
         }
         if (scale) {
           if (any(weights < 0)) {
@@ -291,32 +292,33 @@ data_gr <- function(ranef, data, cov_ranef = NULL) {
   out
 }
 
-data_cs <- function(bterms, data, nlpar = "") {
+data_cs <- function(bterms, data) {
   # prepare data for category specific effects for use in Stan
   # Args:
   #   bterms: object of class brmsterms
   #   data: the data passed by the user
   out <- list()
+  px <- check_prefix(bterms)
   if (length(all_terms(bterms[["cs"]]))) {
-    stopifnot(!nzchar(nlpar))
     Xcs <- get_model_matrix(bterms$cs, data)
-    avoid_auxpars(colnames(Xcs), bterms = bterms)
+    avoid_dpars(colnames(Xcs), bterms = bterms)
     out <- c(out, list(Kcs = ncol(Xcs), Xcs = Xcs))
   }
   out
 }
 
-data_me <- function(bterms, data, nlpar = "") {
+data_me <- function(bterms, data) {
   # prepare data of meausurement error variables for use in Stan
   # Args:
   #   bterms: object of class brmsterms
   #   data: the data passed by the user
   out <- list()
+  px <- check_prefix(bterms)
   meef <- get_me_labels(bterms, data)
   if (length(meef)) {
-    p <- usc(nlpar, "prefix")
+    p <- usc(combine_prefix(px))
     Cme <- get_model_matrix(bterms$me, data)
-    avoid_auxpars(colnames(Cme), bterms = bterms)
+    avoid_dpars(colnames(Cme), bterms = bterms)
     Cme <- Cme[, attr(meef, "not_one"), drop = FALSE]
     Cme <- lapply(seq_len(ncol(Cme)), function(i) Cme[, i])
     if (length(Cme)) {
@@ -337,14 +339,15 @@ data_me <- function(bterms, data, nlpar = "") {
   out
 }
 
-data_gp <- function(bterms, data, nlpar = "", gps = NULL) {
+data_gp <- function(bterms, data, gps = NULL) {
   # prepare data for Gaussian process terms
   # Args:
   #   see data_effects.btl
   out <- list()
+  px <- check_prefix(bterms)
   gpef <- get_gp_labels(bterms)
   if (length(gpef)) {
-    p <- usc(nlpar, "prefix")
+    p <- usc(combine_prefix(px))
     for (i in seq_along(gpef)) {
       pi <- paste0(p, "_", i)
       gp <- eval2(gpef[i])
@@ -388,12 +391,14 @@ data_gp <- function(bterms, data, nlpar = "", gps = NULL) {
   out
 } 
 
-data_offset <- function(bterms, data, nlpar = "") {
+data_offset <- function(bterms, data) {
   # prepare data of offsets for use in Stan
   out <- list()
+  px <- check_prefix(bterms)
   if (is.formula(bterms$offset)) {
+    p <- usc(combine_prefix(px))
     mf <- model.frame(bterms$offset, rm_attr(data, "terms"))
-    out[[paste0("offset", usc(nlpar))]] <- model.offset(mf)
+    out[[paste0("offset", p)]] <- model.offset(mf)
   }
   out
 }
@@ -404,10 +409,10 @@ data_mixture <- function(bterms, prior = brmsprior()) {
   out <- list()
   if (is.mixfamily(bterms$family)) {
     families <- family_names(bterms$family)
-    ap_classes <- auxpar_class(
-      names(c(bterms$auxpars, bterms$fauxpars))
+    dp_classes <- dpar_class(
+      names(c(bterms$dpars, bterms$fdpars))
     )
-    if (!any(ap_classes %in% "theta")) {
+    if (!any(dp_classes %in% "theta")) {
       # estimate mixture probabilities directly
       take <- prior$class == "theta"
       theta_prior <- prior$prior[take]
@@ -426,12 +431,13 @@ data_mixture <- function(bterms, prior = brmsprior()) {
   out
 }
 
-data_prior <- function(prior, data, nlpar = "") {
+data_prior <- function(bterms, data, prior) {
   # data for special priors such as horseshoe and lasso
   out <- list()
-  orig_nlpar <- ifelse(nzchar(nlpar), nlpar, "mu")
-  nlpar <- usc(nlpar)
-  special <- attr(prior, "special")[[orig_nlpar]]
+  px <- check_prefix(bterms)
+  p <- usc(combine_prefix(px))
+  prefix <- combine_prefix(px, keep_mu = TRUE)
+  special <- attr(prior, "special")[[prefix]]
   if (!is.null(special[["hs_df"]])) {
     # data for the horseshoe prior
     hs_obj_names <- paste0("hs_", 
@@ -443,13 +449,13 @@ data_prior <- function(prior, data, nlpar = "") {
     } else {
       hs_data$hs_scale_global <- special$hs_par_ratio / sqrt(nrow(data))
     }
-    names(hs_data) <- paste0(hs_obj_names, nlpar) 
+    names(hs_data) <- paste0(hs_obj_names, p) 
     out <- c(out, hs_data)
   }
   if (!is.null(special[["lasso_df"]])) {
     lasso_obj_names <- paste0("lasso_", c("df", "scale"))
     lasso_data <- special[lasso_obj_names]
-    names(lasso_data) <- paste0(lasso_obj_names, nlpar) 
+    names(lasso_data) <- paste0(lasso_obj_names, p) 
     out <- c(out, lasso_data)
   }
   out
