@@ -101,7 +101,7 @@ make_standata <- function(formula, data, family = gaussian(),
     if (!factors_allowed && !is.numeric(out$Y)) {
       stop2("Family '", family4error, "' requires numeric responses.")
     }
-    # transform and check response variable for different families
+    # transform and check response variables for different families
     regex_pos_int <- "(^|_)(binomial|poisson|negbinomial|geometric)$"
     if (any(grepl(regex_pos_int, families))) {
       if (!all(is_wholenumber(out$Y)) || min(out$Y) < 0) {
@@ -168,20 +168,21 @@ make_standata <- function(formula, data, family = gaussian(),
   # data for various kinds of effects
   only_response <- isTRUE(control$only_response)
   if (!only_response) {
-    ranef <- tidy_ranef(bterms, data, ncat = control$ncat, 
-                        old_levels = control$old_levels)
+    ranef <- tidy_ranef(
+      bterms, data, ncat = control$ncat, 
+      old_levels = control$old_levels
+    )
     args_eff <- nlist(data, ranef, prior, knots, not4stan)
     resp <- bterms$response
     if (length(resp) > 1L && !old_mv) {
       args_eff_spec <- list(
-        x = bterms$auxpars[["mu"]], smooths = control$smooths[["mu"]],
+        x = bterms$dpars[["mu"]], smooths = control$smooths[["mu"]],
         gps = control$gps[["mu"]], Jmo = control$Jmo[["mu"]]
       )
-      bterms$auxpars[["mu"]] <- NULL
+      bterms$dpars[["mu"]] <- NULL
       for (r in resp) {
-        data_eff <- do.call(
-          data_effects, c(args_eff_spec, args_eff, nlpar = r)
-        )
+        args_eff_spec$x$resp <- r
+        data_eff <- do.call(data_effects, c(args_eff_spec, args_eff))
         out <- c(out, data_eff)
       }
       if (is_linear(family)) {
@@ -190,18 +191,19 @@ make_standata <- function(formula, data, family = gaussian(),
         colnames(out$Y) <- resp
       }
     }
-    # data for predictors of auxiliary parameters
-    for (ap in names(bterms$auxpars)) {
+    # data for predictors of distributional parameters
+    for (dp in names(bterms$dpars)) {
       args_eff_spec <- list(
-        x = bterms$auxpars[[ap]], nlpar = ap, 
-        smooths = control$smooths[[ap]],
-        gps = control$gps[[ap]], Jmo = control$Jmo[[ap]]
+        x = bterms$dpars[[dp]], 
+        smooths = control$smooths[[dp]],
+        gps = control$gps[[dp]], 
+        Jmo = control$Jmo[[dp]]
       )
       data_aux_eff <- do.call(data_effects, c(args_eff_spec, args_eff))
       out <- c(out, data_aux_eff)
     }
-    for (ap in names(bterms$fauxpars)) {
-      out[[ap]] <- bterms$fauxpars[[ap]]
+    for (dp in names(bterms$fdpars)) {
+      out[[dp]] <- bterms$fdpars[[dp]]$value
     }
     out <- c(out,
       data_gr(ranef, data, cov_ranef = cov_ranef),
@@ -232,8 +234,8 @@ make_standata <- function(formula, data, family = gaussian(),
               "might be a more efficient choice.")
     }
     if (check_response && any(out$Y > out$trials)) {
-      stop2("Number of trials is smaller than the response ", 
-            "variable would suggest.")
+      stop2("Number of trials is smaller than ", 
+            "the number of events.")
     }
     out$trials <- as.array(out$trials)
   }
