@@ -23,16 +23,19 @@ print.brmssummary <- function(x, digits = 2, ...) {
       x[args] <- x[paste0("n.", args)]
     }
     final_samples <- ceiling((x$iter - x$warmup) / x$thin * x$chains)
-    if (is.numeric(x$loo)) {
-      x$loo <- round(x$loo, digits = digits)
+    valid_ics <- c("loo", "waic", "R2")
+    for (ic in valid_ics) {
+      if (is.numeric(x[[ic]])) {
+        x[[ic]] <- round(x[[ic]], digits = digits)
+      }
     }
-    if (is.numeric(x$waic)) {
-      x$waic <- round(x$waic, digits = digits)
-    }
-    cat(paste0("Samples: ", x$chains, " chains, each with iter = ", x$iter, 
-               "; warmup = ", x$warmup, "; thin = ", x$thin, "; \n",
-               "         total post-warmup samples = ", final_samples, "\n"))
-    cat(paste0("    ICs: LOO = ", x$loo, "; WAIC = ", x$waic, "\n \n"))
+    cat(paste0(
+      "Samples: ", x$chains, " chains, each with iter = ", x$iter, 
+      "; warmup = ", x$warmup, "; thin = ", x$thin, "; \n",
+      "         total post-warmup samples = ", final_samples, "\n"))
+    cat(paste0(
+      "    ICs: LOO = ", x$loo, "; WAIC = ", x$waic, "; R2 = ", x$R2, "\n \n"
+    ))
     
     if (nrow(x$prior)) {
       cat("Priors: \n")
