@@ -53,7 +53,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
       if (is.formula(bterms$adforms$disp)) {
         stop2(err_msg, " when specifying 'disp'.")
       }
-      if (any(c("sigma", "nu") %in% names(bterms$auxpars))) {
+      if (any(c("sigma", "nu") %in% names(bterms$dpars))) {
         stop2(err_msg, " when predicting 'sigma' or 'nu'.")
       }
       str_add(out$data) <- paste0( 
@@ -62,7 +62,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
       if (!is.formula(bterms$adforms$se)) {
         str_add(out$tdataD) <- "  vector[N] se2 = rep_vector(0, N); \n"
       }
-      str_add(out$transD) <- paste0(
+      str_add(out$tparD) <- paste0(
         "  matrix[max(nobs_tg), max(nobs_tg)] res_cov_matrix; \n"                  
       )
       if (Kar && !Kma) {
@@ -75,7 +75,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
         cov_mat_fun <- "arma1"
         cov_mat_args <- "ar[1], ma[1]"
       }
-      str_add(out$transC1) <- paste0(
+      str_add(out$tparC1) <- paste0(
         "  // compute residual covariance matrix \n",
         "  res_cov_matrix = cov_matrix_", cov_mat_fun, 
         "(", cov_mat_args, ", sigma, max(nobs_tg)); \n"
@@ -108,7 +108,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
       if (is.formula(bterms$adforms$se)) {
         stop2(err_msg, " when specifying 'se'.")
       }
-      if (length(bterms$auxpars[["mu"]]$nlpars)) {
+      if (length(bterms$dpars[["mu"]]$nlpars)) {
         stop2(err_msg, " in non-linear models.")
       }
       if (!identical(family$link, "identity")) {
@@ -148,7 +148,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
   if (Karr) {
     # autoregressive effects of the response
     err_msg <- "ARR models are not yet working"
-    if (length(bterms$auxpars[["mu"]]$nlpars)) {
+    if (length(bterms$dpars[["mu"]]$nlpars)) {
       stop2(err_msg, " in non-linear models.")
     }
     if (is_mv) {
@@ -176,7 +176,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
     if (is.formula(bterms$adforms$disp)) {
       stop2(err_msg, " when specifying 'disp'.")
     }
-    if (any(c("sigma", "nu") %in% names(bterms$auxpars))) {
+    if (any(c("sigma", "nu") %in% names(bterms$dpars))) {
       stop2(err_msg, " when predicting 'sigma' or 'nu'.")
     }
     str_add(out$data) <- paste0(
@@ -221,7 +221,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
     if (is_mv) {
       stop2(err_msg, " in multivariate models.")
     }
-    if (length(bterms$auxpars[["mu"]]$nlpars)) {
+    if (length(bterms$dpars[["mu"]]$nlpars)) {
       stop2(err_msg, " in non-linear models.")
     }
     str_add(out$data) <- paste0(
@@ -251,8 +251,10 @@ stan_autocor <- function(autocor, bterms, family, prior) {
       )
       str_add(out$prior) <- paste0(
         stan_prior(prior, class = "car"),
-        "  rcar ~ sparse_car(car, sdcar, Nloc, Nedges, Nneigh,\n",
-        "                    eigenW, edges1, edges2); \n"
+        "  target += sparse_car_lpdf(\n", 
+        "    rcar | car, sdcar, Nloc, Nedges,\n",
+        "    Nneigh, eigenW, edges1, edges2\n", 
+        "  ); \n"
       )
     } else if (identical(autocor$type, "esicar")) {
       str_add(out$fun) <- paste0(
@@ -261,17 +263,19 @@ stan_autocor <- function(autocor, bterms, family, prior) {
       str_add(out$par) <- paste0(
         "  vector[Nloc - 1] zcar; \n"
       )
-      str_add(out$transD) <- paste0(
+      str_add(out$tparD) <- paste0(
         "  vector[Nloc] rcar; \n"                
       )
-      str_add(out$transC1) <- paste0(
+      str_add(out$tparC1) <- paste0(
         "  // apply sum-to-zero constraint \n",
         "  rcar[1:(Nloc - 1)] = zcar; \n",
         "  rcar[Nloc] = - sum(zcar); \n"
       )
       str_add(out$prior) <- paste0(
-        "  rcar ~ sparse_icar(sdcar, Nloc, Nedges, Nneigh,\n",
-        "                     eigenW, edges1, edges2); \n"
+        "  target += sparse_icar_lpdf(\n", 
+        "    rcar | sdcar, Nloc, Nedges,\n",
+        "    Nneigh, eigenW, edges1, edges2\n", 
+        "  ); \n"
       )
     } 
   }
@@ -284,7 +288,7 @@ stan_autocor <- function(autocor, bterms, family, prior) {
     if (is_mv) {
       stop2(err_msg, " in multivariate models.")
     }
-    if (length(bterms$auxpars[["mu"]]$nlpars)) {
+    if (length(bterms$dpars[["mu"]]$nlpars)) {
       stop2(err_msg, " in non-linear models.")
     }
     str_add(out$data) <- 
@@ -303,12 +307,12 @@ stan_autocor <- function(autocor, bterms, family, prior) {
     }
     str_add(out$prior) <- paste0(
       stan_prior(prior, class = "sigmaLL"),
-      "  loclev[1] ~ normal(", center[1], ", sigmaLL); \n",
+      "  target += normal_lpdf(loclev[1] | ", center[1], ", sigmaLL); \n",
       "  for (n in 2:N) { \n",
       "    if (tg[n] == tg[n - 1]) { \n",
-      "      loclev[n] ~ normal(loclev[n - 1], sigmaLL); \n",
+      "      target += normal_lpdf(loclev[n] | loclev[n - 1], sigmaLL); \n",
       "    } else { \n",
-      "      loclev[n] ~ normal(", center[2], ", sigmaLL); \n",
+      "      target += normal_lpdf(loclev[n] | ", center[2], ", sigmaLL); \n",
       "    } \n",
       "  } \n"
     )
@@ -355,13 +359,13 @@ stan_mv <- function(family, response, prior) {
         stan_prior(prior, class = "Lrescor")
       )
       if (family$family == "gaussian") {
-        str_add(out$transD) <- paste0(
+        str_add(out$tparD) <- paste0(
           "  // cholesky factor of residual covariance matrix \n",
           "  cholesky_factor_cov[nresp] LSigma = ",
           "diag_pre_multiply(sigma, Lrescor); \n"
         )
       } else if (family$family == "student") {
-        str_add(out$transD) <- paste0(
+        str_add(out$tparD) <- paste0(
           "  // residual covariance matrix \n",
           "  cov_matrix[nresp] Sigma",
           " = multiply_lower_tri_self_transpose(", 
@@ -431,11 +435,11 @@ stan_ordinal <- function(family, prior, cs, disc) {
     } else if (threshold == "equidistant") {
       str_add(out$par) <- paste0(
         "  real temp_Intercept1;  // threshold 1 \n",
-        "  real", if (family == "cumulative") "<lower=0>",
+        "  real", prior[prior$class == "delta", "bound"],
         " delta;  // distance between thresholds \n"
       )
-      str_add(out$transD) <- intercept
-      str_add(out$transC1) <- paste0(
+      str_add(out$tparD) <- intercept
+      str_add(out$tparC1) <- paste0(
         "  // compute equidistant thresholds \n",
         "  for (k in 1:(ncat - 1)) { \n",
         "    temp_Intercept[k] = temp_Intercept1 + (k - 1.0) * delta; \n",
@@ -515,9 +519,7 @@ stan_ordinal <- function(family, prior, cs, disc) {
           )
         }
       }
-      str_add(out$fun) <- paste(
-        "    return categorical_lpmf(y | p); \n   } \n"
-      )
+      str_add(out$fun) <- "    return categorical_lpmf(y | p); \n   } \n"
     }
   }
   out
@@ -573,7 +575,7 @@ stan_families <- function(family, bterms) {
     # as suggested by Stephen Martin use sigma and mu of CP 
     # but the skewness parameter alpha of DP
     str_add(out$tdataD) <- "  real sqrt_2_div_pi = sqrt(2 / pi()); \n"
-    ap_names <- names(bterms$auxpars)
+    ap_names <- names(bterms$dpars)
     for (i in which(families %in% "skew_normal")) {
       id <- ifelse(length(families) == 1L, "", i)
       ns <- ifelse(paste0("sigma", id) %in% ap_names, "[n]", "")
@@ -609,7 +611,7 @@ stan_families <- function(family, bterms) {
       "  #include 'fun_gen_extreme_value.stan' \n",
       "  #include 'fun_scale_xi.stan' \n"
     )
-    ap_names <- c(names(bterms$auxpars), names(bterms$fauxpars))
+    ap_names <- c(names(bterms$dpars), names(bterms$fdpars))
     for (i in which(families %in% "gen_extreme_value")) {
       id <- ifelse(length(families) == 1L, "", i)
       xi <- paste0("xi", id)
@@ -618,7 +620,7 @@ stan_families <- function(family, bterms) {
            "  real ", xi, ";  // scaled shape parameter \n"
         )
         sigma <- paste0("sigma", id)
-        v <- ifelse(sigma %in% names(bterms$auxpars), "_vector", "")
+        v <- ifelse(sigma %in% names(bterms$dpars), "_vector", "")
         args <- sargs(paste0("temp_", xi), "Y", paste0("mu", id), sigma)
         str_add(out$modelC) <- paste0(
            "  ", xi, " = scale_xi", v, "(", args, "); \n"
@@ -634,10 +636,10 @@ stan_mixture <- function(bterms, prior) {
   out <- list()
   if (is.mixfamily(bterms$family)) {
     nmix <- length(bterms$family$mix)
-    theta_pred <- grepl("^theta", names(bterms$auxpars))
-    theta_pred <- bterms$auxpars[theta_pred]
-    theta_fix <- grepl("^theta", names(bterms$fauxpars))
-    theta_fix <- bterms$fauxpars[theta_fix]
+    theta_pred <- grepl("^theta", names(bterms$dpars))
+    theta_pred <- bterms$dpars[theta_pred]
+    theta_fix <- grepl("^theta", names(bterms$fdpars))
+    theta_fix <- bterms$fdpars[theta_fix]
     def_thetas <- collapse(
       "  real<lower=0,upper=1> theta", 1:nmix, ";",
       "  // mixing proportion \n"
@@ -646,7 +648,7 @@ stan_mixture <- function(bterms, prior) {
       if (length(theta_pred) != nmix - 1) {
         stop2("Can only predict all but one mixing proportion.")
       }
-      missing_id <- setdiff(1:nmix, auxpar_id(names(theta_pred)))
+      missing_id <- setdiff(1:nmix, dpar_id(names(theta_pred)))
       str_add(out$modelD) <- paste0(
         "  vector[N] theta", missing_id, " = rep_vector(0, N); \n",                   
         "  real log_sum_exp_theta; \n"      
@@ -680,9 +682,9 @@ stan_mixture <- function(bterms, prior) {
         "  // mixing proportions \n"
       )
       str_add(out$prior) <- paste0(
-        "  theta ~ dirichlet(con_theta); \n"                
+        "  target += dirichlet_lpdf(theta | con_theta); \n"                
       )
-      str_add(out$transD) <- paste0(
+      str_add(out$tparD) <- paste0(
         "  // mixing proportions \n",                
         collapse(
           "  real<lower=0,upper=1> theta", 1:nmix, 
@@ -793,7 +795,7 @@ stan_misc_functions <- function(family, prior, kronecker) {
 }
 
 stan_prior <- function(prior, class, coef = "", group = "", 
-                       nlpar = "", prefix = "", suffix = "",
+                       px = list(), prefix = "", suffix = "", 
                        wsp = 2, matrix = FALSE) {
   # Define priors for parameters in Stan language
   # Args:
@@ -811,61 +813,71 @@ stan_prior <- function(prior, class, coef = "", group = "",
   #   A character strings in stan language that defines priors 
   #   for a given class of parameters. If a parameter has has 
   #   no corresponding prior in prior, an empty string is returned.
-  wsp <- collapse(rep(" ", wsp))
+  tp <- tp(wsp)
+  wsp <- wsp(nsp = wsp)
   prior_only <- identical(attr(prior, "sample_prior"), "only")
-  keep <- prior$class == class & 
-    prior$coef %in% c(coef, "") & prior$group %in% c(group, "")
+  prior <- subset2(prior, 
+    class = class, coef = c(coef, ""), group = c(group, "")
+  )
   if (class %in% c("sd", "cor")) {
     # only sd and cor parameters have global priors
-    keep <- keep & prior$nlpar %in% c(nlpar, "") 
+    px_tmp <- lapply(px, function(x) c(x, ""))
+    prior <- subset2(prior, ls = px_tmp)
   } else {
-    keep <- keep & prior$nlpar %in% nlpar
+    prior <- subset2(prior, ls = px)
   }
-  prior <- prior[keep, ]
   if (!nchar(class) && nrow(prior)) {
     # unchecked prior statements are directly passed to Stan
     return(collapse(wsp, prior$prior, "; \n"))
   } 
   
-  unlpar <- unique(nlpar)
-  if (length(unlpar) > 1L) {
+  px <- as.data.frame(px)
+  upx <- unique(px)
+  if (nrow(upx) > 1L) {
     # can only happen for SD parameters of the same ID
-    base_prior <- rep(NA, length(unlpar))
-    for (i in seq_along(unlpar)) {
-      nlpar_prior <- prior[prior$nlpar %in% c("", unlpar[i]), ]
-      base_prior[i] <- stan_base_prior(nlpar_prior)
+    base_prior <- rep(NA, nrow(upx))
+    for (i in seq_len(nrow(upx))) {
+      sub_upx <- lapply(upx[i, ], function(x) c(x, ""))
+      sub_prior <- subset2(prior, ls = sub_upx) 
+      base_prior[i] <- stan_base_prior(sub_prior)
     }
     if (length(unique(base_prior)) > 1L) {
       # define prior for single coefficients manually
       # as there is not single base_prior anymore
-      take <- match(prior[nzchar(prior$coef), "nlpar"], unlpar)
+      prior_of_coefs <- prior[nzchar(prior$coef), vars_prefix()]
+      take <- match_rows(prior_of_coefs, upx)
       prior[nzchar(prior$coef), "prior"] <- base_prior[take]
     }
     base_prior <- base_prior[1]
+    bound <- ""
   } else {
     base_prior <- stan_base_prior(prior)
+    bound <- prior[!nzchar(prior$coef), "bound"]
   }
   
-  individual_prior <- function(i, max_index) {
+  individual_prior <- function(i, prior, max_index) {
     # individual priors for each parameter of a class
     if (max_index > 1L || matrix) {
       index <- paste0("[", i, "]")      
     } else {
       index <- ""
     }
-    if (length(nlpar) > 1L) {
-      prior <- prior[prior$nlpar == nlpar[i], ]
+    if (nrow(px) > 1L) {
+      prior <- subset2(prior, ls = px[i, ])
     }
     uc_prior <- prior$prior[match(coef[i], prior$coef)]
     if (!is.na(uc_prior) & nchar(uc_prior)) { 
       # user defined prior for this parameter
       coef_prior <- uc_prior
-    } else { # base prior for this parameter
+    } else { 
+      # base prior for this parameter
       coef_prior <- base_prior 
     }  
-    if (nchar(coef_prior) > 0) {  
+    if (nzchar(coef_prior)) {  
       # implies a proper prior
-      out <- paste0(wsp, class, index, " ~ ", coef_prior, "; \n")
+      pars <- paste0(class, index)
+      out <- stan_target_prior(coef_prior, pars, bound = bound)
+      out <- paste0(tp, out, "; \n")
     } else {
       # implies an improper flat prior
       out <- ""
@@ -877,47 +889,25 @@ stan_prior <- function(prior, class, coef = "", group = "",
   class <- paste0(prefix, class, suffix)
   if (any(with(prior, nchar(coef) & nchar(prior)))) {
     # generate a prior for each coefficient
-    out <- sapply(seq_along(coef), individual_prior, 
-                  max_index = length(coef))
+    out <- sapply(
+      seq_along(coef), individual_prior, 
+      prior = prior, max_index = length(coef)
+    )
   } else if (nchar(base_prior) > 0) {
     if (matrix) {
       class <- paste0("to_vector(", class, ")")
     }
-    out <- paste0(wsp, class, " ~ ", base_prior, "; \n")
+    out <- stan_target_prior(
+      base_prior, class, ncoef = length(coef), bound = bound
+    )
+    out <- paste0(tp, out, "; \n")
   } else {
     out <- ""
   }
-  p <- usc(unlpar)
-  if (all(class == paste0("b", p))) {
-    stopifnot(length(unlpar) == 1L)
-    # add horseshoe and lasso shrinkage priors
-    orig_nlpar <- ifelse(nzchar(unlpar), unlpar, "mu")
-    special <- attributes(prior)$special[[orig_nlpar]]
-    special_priors <- NULL
-    if (!is.null(special$hs_df)) {
-      local_args <- paste0("0.5 * hs_df", p)
-      local_args <- sargs(local_args, local_args)
-      global_args <- paste0("0.5 * hs_df_global", p)
-      global_args <- sargs(global_args, global_args)
-      c2_args <- paste0("0.5 * hs_df_slab", p)
-      c2_args <- sargs(c2_args, c2_args)
-      str_add(special_priors) <- paste0(
-        "  zb", p, " ~ normal(0, 1); \n",
-        "  hs_local", p, "[1] ~ normal(0, 1); \n",
-        "  hs_local", p, "[2] ~ inv_gamma(", local_args, "); \n",
-        "  hs_global", p, "[1] ~ normal(0, 1); \n",
-        "  hs_global", p, "[2] ~ inv_gamma(", global_args, "); \n",
-        "  hs_c2", p, " ~ inv_gamma(", c2_args, "); \n"
-      )
-    }
-    if (!is.null(special$lasso_df)) {
-      str_add(special_priors) <- paste0(
-        "  lasso_inv_lambda", p, " ~ chi_square(lasso_df", p, "); \n"
-      )
-    }
-    out <- c(special_priors, out) 
-  }
-  out <- collapse(out)
+  special_prior <- stan_special_prior(
+    class, prior, ncoef = length(coef), px = px
+  )
+  out <- collapse(c(out, special_prior))
   if (prior_only && nzchar(class) && !nchar(out)) {
     stop2("Sampling from priors is not possible as ", 
           "some parameters have no proper priors. ",
@@ -931,26 +921,106 @@ stan_base_prior <- function(prior) {
   # Args:
   #   prior: a prior.frame
   stopifnot(length(unique(prior$class)) <= 1L) 
-  igroup <- which(with(prior, !nchar(coef) & nchar(group) & nchar(prior)))
-  inlpar <- which(with(prior, !nchar(coef) & nchar(nlpar) & nchar(prior)))
-  iclass <- which(with(prior, !nchar(coef) & !nchar(group) & nchar(prior)))
-  if (length(igroup)) {  
-    # if there is a global prior for this group
-    base_prior <- prior[igroup, "prior"]
-  } else if (length(inlpar)) {
-    # if there is a global prior for this non-linear parameter
-    base_prior <- prior[inlpar, "prior"]
-  } else if (length(iclass)) {  
-    # if there is a global prior for this class
-    base_prior <- prior[iclass, "prior"]
-  } else {  
-    # no proper prior for this class
-    base_prior <- ""
+  prior <- prior[with(prior, !nzchar(coef) & nzchar(prior)), ]
+  vars <- c("group", "nlpar", "dpar", "resp", "class")
+  i <- 1
+  found <- FALSE
+  base_prior <- ""
+  take <- rep(FALSE, nrow(prior))
+  while (!found && i <= length(vars)) {
+    take <- nzchar(prior[[vars[i]]]) & !take
+    if (any(take)) {
+      base_prior <- prior[take, "prior"]
+      found <- TRUE
+    }
+    i <- i + 1
   }
   stopifnot(length(base_prior) == 1L)
   base_prior
 }
 
+stan_target_prior <- function(prior, par, ncoef = 1, bound = "") {
+  prior_name <- get_matches("^[^\\(]+\\(", prior, simplify = FALSE)
+  for (i in seq_along(prior_name)) {
+    if (length(prior_name[[i]]) != 1L) {
+      stop2("The prior '", prior[i], "' is invalid.")
+    }
+  }
+  prior_name <- unlist(prior_name)
+  prior_name <- substr(prior_name, 1, nchar(prior_name) - 1)
+  prior_args <- rep(NA, length(prior))
+  for (i in seq_along(prior)) {
+    prior_args[i] <- sub(
+      paste0("^", prior_name[i], "\\("), "", prior[i]
+    )
+  }
+  out <- paste0(prior_name, "_lpdf(", par, " | ", prior_args)
+  par_class <- unique(get_matches("^[^_]+", par))
+  par_bound <- par_bounds(par_class, bound)
+  prior_bound <- prior_bounds(prior_name)
+  trunc_lb <- is.character(par_bound$lb) || par_bound$lb > prior_bound$lb
+  trunc_ub <- is.character(par_bound$ub) || par_bound$ub < prior_bound$ub
+  if (trunc_lb || trunc_ub) {
+    wsp <- wsp(nsp = 4)
+    if (trunc_lb && !trunc_ub) {
+      str_add(out) <- paste0(
+        "\n", wsp, "- ", ncoef, " * ", prior_name, "_lccdf(", 
+        par_bound$lb, " | ", prior_args
+      )
+    } else if (!trunc_lb && trunc_ub) {
+      str_add(out) <- paste0(
+        "\n", wsp, "- ", ncoef, " * ", prior_name, "_lcdf(", 
+        par_bound$ub, " | ", prior_args
+      )
+    } else if (trunc_lb && trunc_ub) {
+      str_add(out) <- paste0(
+        "\n", wsp, "- ", ncoef, " * log_diff_exp(", 
+        prior_name, "_lcdf(", par_bound$ub, " | ", prior_args, ", ",
+        prior_name, "_lcdf(", par_bound$lb, " | ", prior_args, ")"
+      )
+    }
+  }
+  out
+}
+
+stan_special_prior <- function(class, prior, ncoef, px = list()) {
+  # add special priors such as horseshoe and lasso
+  out <- ""
+  p <- usc(combine_prefix(px))
+  if (all(class == paste0("b", p))) {
+    stopifnot(length(p) == 1L)
+    tp <- tp()
+    # add horseshoe and lasso shrinkage priors
+    prefix <- combine_prefix(px, keep_mu = TRUE)
+    special <- attributes(prior)$special[[prefix]]
+    if (!is.null(special$hs_df)) {
+      local_args <- paste0("0.5 * hs_df", p)
+      local_args <- sargs(local_args, local_args)
+      global_args <- paste0("0.5 * hs_df_global", p)
+      global_args <- sargs(global_args, global_args)
+      c2_args <- paste0("0.5 * hs_df_slab", p)
+      c2_args <- sargs(c2_args, c2_args)
+      wsp <- wsp(nsp = 4)
+      str_add(out) <- paste0(
+        tp, "normal_lpdf(zb", p, " | 0, 1); \n",
+        tp, "normal_lpdf(hs_local", p, "[1] | 0, 1)\n", 
+        wsp, "- ", ncoef, " * log(0.5); \n",
+        tp, "inv_gamma_lpdf(hs_local", p, "[2] | ", local_args, "); \n",
+        tp, "normal_lpdf(hs_global", p, "[1] | 0, 1)\n", 
+        wsp, "- 1 * log(0.5); \n",
+        tp, "inv_gamma_lpdf(hs_global", p, "[2] | ", global_args, "); \n",
+        tp, "inv_gamma_lpdf(hs_c2", p, " | ", c2_args, "); \n"
+      )
+    }
+    if (!is.null(special$lasso_df)) {
+      str_add(out) <- paste0(
+        tp, "chi_square_lpdf(lasso_inv_lambda", p, " | lasso_df", p, "); \n"
+      )
+    }
+  }
+  out
+}
+
 stan_rngprior <- function(sample_prior, prior, par_declars,
                           gen_quantities, prior_special) {
   # stan code to sample from priors seperately
@@ -966,15 +1036,20 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
   #   a character string containing the priors to be sampled from in stan code
   out <- list()
   if (identical(sample_prior, "yes")) {
-    prior <- gsub(" ", "", paste0("\n", prior))
-    pars <- get_matches("\\\n[^~]+", prior)
-    pars <- gsub("\\\n|to_vector\\(|\\)", "", pars)
-    regex <- "^(z|zs|zb|zgp|Xme|hs)_?|^increment_log_prob\\(|^target ?(\\+=)"
+    prior <- strsplit(gsub(" |\\n", "", prior), ";")[[1]]
+    prior <- prior[nzchar(prior)]
+    pars <- get_matches("_lpdf\\([^|]+", prior, first = TRUE)
+    pars <- gsub("^_lpdf\\(|to_vector\\(|\\)$", "", pars)
+    regex <- "^(z|zs|zb|zgp|Xme|hs)_?|^increment_log_prob\\("
     take <- !grepl(regex, pars)
-    pars <- rename(pars[take], symbols = c("^L_", "^Lrescor"), 
-                   subs = c("cor_", "rescor"), fixed = FALSE)
-    dis <- gsub("~", "", get_matches("~[^\\(]+", prior))[take]
-    args <- get_matches("\\([^;~]+\\);", prior)[take]
+    pars <- rename(
+      pars[take], symbols = c("^L_", "^Lrescor"), 
+      subs = c("cor_", "rescor"), fixed = FALSE
+    )
+    dis <- get_matches("target\\+=[^\\(]+", prior, first = TRUE)
+    dis <- gsub("target\\+=|_lpdf", "", dis[take])
+    args <- get_matches("\\|[^$\\|]+\\)($|-)", prior, first = TRUE)
+    args <- gsub("\\||(;|-)$", "", args[take])
     type <- rep("real", length(pars))
     
     # rename parameters containing indices
@@ -988,8 +1063,7 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
     # special treatment of lkj_corr_cholesky priors
     args <- ifelse(
       grepl("corr_cholesky$", dis), 
-      paste0("(2,", substr(args, 2, nchar(args) - 1), "[1, 2];"),
-      args
+      paste0("2,", args, "[1, 2]"), args
     )
     dis <- sub("corr_cholesky$", "corr", dis)
     
@@ -1028,8 +1102,8 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
       str_add(out$model) <- paste0(
         "  // additionally draw samples from priors\n",
         collapse(
-          "  prior_", pars[has_bounds] ," ~ ",
-          dis[has_bounds], args[has_bounds], "\n"
+          "  target += ", dis[has_bounds], "_lpdf(",
+          "prior_", pars[has_bounds], " | ", args[has_bounds], "; \n"
         )
       )
     }
@@ -1038,11 +1112,11 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
       # use parameters sampled from priors for use in other priors
       spars <- NULL
       # cannot sample from the horseshoe prior anymore as of brms 1.5.0
-      lasso_nlpars <- nzchar(ulapply(prior_special, "[[", "lasso_df"))
-      lasso_nlpars <- names(prior_special)[lasso_nlpars]
-      lasso_nlpars <- usc(ulapply(lasso_nlpars, check_nlpar))
-      if (length(lasso_nlpars)) {
-        spars <- c(spars, paste0("lasso_inv_lambda", lasso_nlpars))
+      lasso_prefix <- nzchar(ulapply(prior_special, "[[", "lasso_df"))
+      lasso_prefix <- names(prior_special)[lasso_prefix]
+      lasso_prefix <- usc(sub("^mu(_|$)", "", lasso_prefix))
+      if (length(lasso_prefix)) {
+        spars <- c(spars, paste0("lasso_inv_lambda", lasso_prefix))
       }
       if (length(spars)) {
         bpars <- grepl("^b(|mo|cs|me)(_|$)", pars)
@@ -1053,7 +1127,7 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
         "  // additionally draw samples from priors \n",
         collapse(
           "  ", types[no_bounds], " prior_", pars[no_bounds], 
-          " = ", dis[no_bounds], "_rng", args[no_bounds], " \n"
+          " = ", dis[no_bounds], "_rng(", args[no_bounds], "; \n"
         )
       )
     }
@@ -1126,7 +1200,7 @@ stan_has_built_in_fun <- function(family) {
   #   family: a list with elements 'family' and 'link'
   stopifnot(all(c("family", "link") %in% names(family)))
   link <- family$link
-  par <- family$par
+  dpar <- family$dpar
   family <- family$family
   log_families <- c(
     "poisson", "negbinomial", "geometric", 
@@ -1140,7 +1214,7 @@ stan_has_built_in_fun <- function(family) {
   isTRUE(
     family %in% log_families && link == "log" ||
     family %in% logit_families && link == "logit" ||
-    isTRUE(par %in% c("zi", "hu")) && link == "logit"
+    isTRUE(dpar %in% c("zi", "hu")) && link == "logit"
   )
 }
 
