@@ -181,7 +181,7 @@ test_that("make_standata rejects incorrect addition terms", {
   expect_error(make_standata(y | cens(c) ~ 1, data = dat))
   expect_error(make_standata(z | trials(t) ~ 1, data = dat, 
                              family = "binomial"),
-               "Number of trials is smaller than the response variable")
+               "Number of trials is smaller than the number of events")
 })
 
 test_that("make_standata handles multivariate models", {
@@ -296,6 +296,16 @@ test_that("make_standata correctly prepares data for non-linear models", {
   sdata <- make_standata(bf(y ~ a - b^z, flist = flist, nl = TRUE), 
                          data = data, control = list(not4stan = TRUE))
   expect_equal(colnames(sdata$C), "z")
+  
+  bform <- bf(y ~ x) + 
+    nlf(sigma ~ a1 * exp(-x/(a2 + z)),
+        a1 ~ 1, a2 ~ z + (x|g)) +
+    lf(alpha ~ x)
+  sdata <- make_standata(bform, data, family = skew_normal())
+  sdata_names <- c("C_sigma_1", "C_sigma_2", "X_sigma_a2", "Z_1_sigma_a2_1")
+  expect_true(
+    all(sdata_names %in% names(sdata))
+  )
 })
 
 test_that("make_standata correctly prepares data for monotonic effects", {
@@ -309,7 +319,7 @@ test_that("make_standata correctly prepares data for monotonic effects", {
   expect_equal(sdata$con_simplex_1, rep(1, 3))
   
   prior <- set_prior("dirichlet(1:3)", coef = "x1", 
-                     class = "simplex", nlpar = "sigma")
+                     class = "simplex", dpar = "sigma")
   sdata <- make_standata(bf(y ~ 1, sigma ~ mono(x1)), 
                          data = data, prior = prior)
   expect_equal(sdata$con_simplex_sigma_1, 1:3)
@@ -463,7 +473,7 @@ test_that("make_standata allows fixed auxiliary parameters", {
   expect_equal(make_standata(bf(y ~ 1, nu = 3), dat, student())$nu, 3)
   expect_equal(make_standata(y ~ 1, dat, acat())$disc, 1)
   expect_error(make_standata(bf(y ~ 1, bias = 0.5), dat),
-               "Invalid auxiliary parameters: 'bias'")
+               "Invalid fixed parameters: 'bias'")
 })
 
 test_that("make_standata correctly includes offsets", {
